/**************************************************************

MapDisplay.h

An object to display data and objects from a 
MilMap object.

**************************************************************/

#import <space.h>
#import <tkobjc/Raster.h>
#import "MilMap.h"
#import "BattleArea.h"
#import <mylib/TkMessage.h>
#import <tkobjc/internal.h>

void
tkobjc_draw_text (Raster *raster,
                    int x0, int y0,
                    const char * text, 
                    Color color);

@interface MapDisplay: Raster {
  id buttonReleaseClient, doubleClickClient;
  SEL buttonReleaseSel, doubleClickSel;
  id <Discrete2d> elevations;  // a grid of interpolated elevations
                               //   from the map
  int elevationsSizeX;         // size of elevations display grid X
  int elevationsSizeY;         // size of elevations display grid Y
  id <Discrete2d> objects;     // displays objects on the grid
  id <Value2dDisplay> elevDisplay; // Value 2d display object
  id milMap;                   // The map object to display
  double metersPerSquare;      // Size of cells in display grid
  double xsw, ysw;             // SW corner of map display area
}

-createEnd;
-cleanCopyIn: aZone And: p;    // makes copy with nothing in objects
-setMap: (id) aMap andMetersPerSquare: (double) meters;
                               // Sets the map to display and number of 
                               //   meters in squares of display grid
//-setColormap: (Colormap*) cMap;
                               // Sets the color map for the elevation
                               //   display.  The colors 0 - 63 in this
                               //   color map will be used to display
                               //   elevation]
-displayElevations;            // draw elevation on raster display
-setObjects: o;                // sets new object grid for map
-getObjects;                   // returns discrete2d storing objects
                               //   on the map
-getMilMap;                    // returns milMap object on display
-(double) getMetersPerSquare;  // returns number of meters in display square
-drawText: (const char *) text AtX: (int) x Y:(int) y Color: (Color) c;

// drawing routines which take double value map coords in meters
-dblDrawText: (const char *) text AtX:(double) x Y:(double) y Color: (Color) c;
- dblDrawPointX: (double)x Y: (double)y Color: (Color)c;
- drawPointX: (double)x Y: (double)y Color: (Color)c;
- dblFillRectangleX0: (double)x0 Y0: (double)y0 
               X1: (double)x1 Y1: (double)y1 Color: (Color)c;

// - dblDraw: (id <XDrawer>)xd X: (double)x Y: (double)y;

-dblDrawLineFromX: (double) x1 Y: (double) y1 
	   ToX: (double) x2 Y: (double) y2 Color: (Color) c;
-dblEllipseX0: (double)x0 Y0: (double)y0 X1: (double)x1 Y1: (double)y1 
    Width: (unsigned)penWidth Color: (Color)c;
-dblPutObject: anObject atX: (double) x Y: (double) y; 
                              // places object on map lattice
-dblRestoreElevationAtX: (double) x Y: (double) y;
                              // restores elevation color to a location

// conversions from map coordinates to raster coordinates
-(int) convertCoordX: (double) x;
-(int) convertCoordY: (double) y;

// conversions from raster coordinates to map coordinates
-(double) convertGridX: (int) x;
-(double) convertGridY: (int) y;

// get size of display raster
-(unsigned) getWidth;
-(unsigned) getHeight;

// accept inputs for polygon areas
-acceptInputFor: area Called:(const char*)areaName UsingWidget: inputWidget;

-setButtonReleaseClient: c Message: (SEL)s;
-setDoubleClickClient: c Message: (SEL)s;
-handleButtonRelease: (int)n X: (int)x Y: (int)y;
-handleDoubleClick: (int)n X: (int)x Y: (int)y;
-displayAll;
-getObjectNearX: (int) x Y: (int) y Error: (int) e;
// checks +- e units from x,y until an object is found


@end

























