/**************************************************************

MapDisplay.m

An object to display elevation data from a MilMap object.

**************************************************************/

#import "MapDisplay.h"

#ifdef _WIN32
// Get a definition of timeval, but avoid redefinition of timezone.
#define timezone timezone__
#include <sys/time.h>
#undef timezone
// Undo X11 definition
#undef Status
#define Rectangle Win32Rectangle
#include <tkobjc/win32dib.h>
#undef BOOL
#define BOOL BOOL_
//#include "tk/tkInt.h"
#undef BOOL

#define Arguments Win32Arguments
#define Colormap X11Colormap
//#include "tk/tkWinInt.h"
#undef Colormap
#undef Arguments
#undef Rectangle
#endif


@implementation MapDisplay 

-createEnd {
  [super createEnd];
  [globalTkInterp eval: 
    "bind %s <ButtonRelease-1> { %s handleButtonRelease: %s X: %s Y: %s }",
		  widgetName, [self getObjectName], "%b", "%x", "%y"];
  [globalTkInterp eval: 
    "bind %s <Double-Button-1> { %s handleDoubleClick: %s X: %s Y: %s }",
		  widgetName, [self getObjectName], "%b", "%x", "%y"];
  return self;
}

-cleanCopyIn: aZone And: p {  // makes copy with nothing in objects
  MapDisplay  *newCopy;    // new map display to create
  id newObjects;           // new object display
 
  newCopy = [MapDisplay createParent: p];
  newCopy->elevations = elevations;
  newCopy->elevationsSizeX = elevationsSizeX;
  newCopy->elevationsSizeY = elevationsSizeY;
  newCopy->metersPerSquare = metersPerSquare;
  newCopy->xsw = xsw;
  newCopy->ysw = ysw;
  newCopy->milMap = milMap;
  newCopy->colormap = colormap;
  newCopy->width = width;
  newCopy->height = height;

  newObjects = [Discrete2d createBegin: aZone];
  [newObjects setSizeX: [objects getSizeX] Y: [objects getSizeY]];
  newObjects = [newObjects createEnd];
  [newCopy setObjects: newObjects];
  return newCopy;
}
 
-setObject: o {                // sets new object grid for map
    objects = o;
    return self;
  }
// Sets the map to display and number of meters in squares 
//  of display grid
-setMap: (id) aMap andMetersPerSquare: (double) meters {
  double mapSizeX;         // size of map in meters
  double mapSizeY;         // size of map in meters
  double minElev;          // min elevation on the map
  double maxElev;          // max elevation on the map
  double temp;             // temporary storage value
  int x,y;
  BOOL hasFeatureData;     // flag if map has feature data
  BOOL plotFeature;        // flag to plot feature as opposed to elevation
  int f;                   // feature data


  milMap = aMap;
  metersPerSquare = meters;
  
  elevations = [Discrete2d createBegin: [self getZone]];
  objects = [Discrete2d createBegin: [self getZone]];
  mapSizeX = [milMap getXcells] * [milMap getCellSize];
  mapSizeY = [milMap getYcells] * [milMap getCellSize];
  elevationsSizeX = (int)(mapSizeX/metersPerSquare);
  elevationsSizeY = (int)(mapSizeY/metersPerSquare); 
  fprintf(stdout, "Map size is %f by %f\n"
                  "Raster size is %d by %d with %f meterPerSquare\n", 
	  mapSizeX, mapSizeY, elevationsSizeX, elevationsSizeY,
	  metersPerSquare);
  [elevations setSizeX: elevationsSizeX
       Y: elevationsSizeY]; 
  [objects setSizeX: elevationsSizeX
       Y: elevationsSizeY]; 
  elevations = [elevations createEnd];
  objects = [objects createEnd];

  minElev = (double)[milMap getMinElev];
  maxElev = (double)[milMap getMaxElev];
  xsw = [milMap getSWCornerX];
  ysw = [milMap getSWCornerY];
  fprintf(stdout, "SW corner is %f, %f\n", xsw, ysw);
  hasFeatureData = [milMap getHasFeatureData];

  // Fill the elevations object with values
  for(y = 0; y < elevationsSizeY; y++) {
    for (x = 0; x < elevationsSizeX; x++) {
      plotFeature = 0;
      if (hasFeatureData) {     
	f = [milMap getFeatureAtX: xsw + ((double)x * metersPerSquare) 
             Y: ysw +(((double)elevationsSizeY - 
		       (double)y -1) * metersPerSquare)];
	if (f % 10 > 0) {  // plot all linear features
	  plotFeature = 1;
	  [elevations putValue: (f % 10) + 230 atX: x Y: y];
	}
	else if (x % 3 == 0 && y % 3 == 0 && (int)(f/10) > 0) {
	  // plot area feature at 5x5 intervals
	  plotFeature = 1;
	  [elevations putValue: (int)(f / 10) + 220 atX: x Y: y];
	}
      }
      if (plotFeature == 0) {
	        
	temp = [milMap interpElevAtX: xsw + ((double)x * metersPerSquare) 
             Y: ysw +(((double)elevationsSizeY - 
		       (double)y -1) * metersPerSquare)];
	/*if (x == 300) {
	  fprintf(stdout, "x,y are %f, %f for cell %d, %d\n", 
	       xsw + ((double)x * metersPerSquare),
	       ysw +(((double)elevationsSizeY - 
	       (double)y -1) * metersPerSquare), x,y);
	       }*/
	// convert elevation to a color in the range of 0 to 63
	temp = ((temp - minElev) / (maxElev - minElev)) * 63;
	[elevations putValue: (int) temp atX: x Y: y];
      }
    }
  }

  [self setWidth: elevationsSizeX
	       Height: elevationsSizeY];

  // Create Value2dDisplay object for the map
  elevDisplay = [Value2dDisplay createBegin: [self getZone]];
  [elevDisplay setDisplayWidget: self colormap: colormap];
  [elevDisplay setDiscrete2dToDisplay: elevations];
  elevDisplay = [elevDisplay createEnd];
  return self;
}
  
   
// Sets the color map for the elevation display.  The colors 0 - 63 
//   in this color map will be used to display elevation  
//-setColormap: cMap {
//  id elevs;       // for debugging
//  elevs = elevations;
//  [super setColormap: cMap];
//

//  return self;
//}

-displayElevations {               // draw elevation on raster display
  [elevDisplay display];
  [self drawGridLines];
  [self drawSelf];
  return self;
}

-(double) getMetersPerSquare { // returns number of meters in display square
  return metersPerSquare;
}

-setObjects: o {               // sets new object grid for map
  objects = o;
  return self;
}

-getObjects {  // returns Discrete2d of objects for display
  return objects;
}

-getMilMap {                   // returns milMap object on display
  return milMap;
}

// drawing routines which take double value map coords in meters
- dblDrawPointX: (double)x Y: (double)y Color: (Color)c {
  [self drawPointX: [self convertCoordX: x] 
  Y: [self convertCoordY: y] Color: c];
  return self;
}

- drawPointX: (int)x Y: (int)y Color: (Color)c {
  [self fillRectangleX0: x Y0: y X1: x+1  Y1:y+1 Color: c];
  return self;
}
  
-dblEllipseX0: (double)x0 Y0: (double)y0 X1: (double)x1 Y1: (double)y1 
	Width: (unsigned)penWidth Color: (Color)c {
  [self ellipseX0: [self convertCoordX: x0]
	Y0: [self convertCoordY: y1]
	X1: [self convertCoordX: x1]
	Y1: [self convertCoordY: y0] Width: penWidth Color: c];
  return self;
}
	
- dblFillRectangleX0: (double)x0 Y0: (double)y0 
    X1: (double)x1 Y1: (double)y1 Color: (Color)c {
      [self fillRectangleX0: [self convertCoordX: x0]
	    Y0: [self convertCoordY: y1]
            X1: [self convertCoordX: x1]
            Y1: [self convertCoordY: y0] Color: c];
      return self;
}

//#ifndef _WIN32
//- dblDraw: (id <XDrawer>)xd X: (double)x Y: (double)y {
//   [self draw: xd X: [self convertCoordX: x] Y: [self convertCoordY: y]];
//   return self;
//}
//#endif

-drawText: (const char *) text AtX: (int) x Y:(int) y Color: (Color) c {
#ifndef _WIN32
  tkobjc_draw_text (self,
                   x, y,
                   text, 
                   c);
#endif
  return self;
}

-dblDrawText:(const char *)text AtX:(double)x Y:(double) y Color:(Color) c {
  [self drawText: text AtX: [self convertCoordX: x]
	 Y: [self convertCoordY: y] Color: c];
  return self;
}

-dblDrawLineFromX: (double) x1 Y: (double) y1 
	      ToX: (double) x2 Y: (double) y2 Color: (Color) c {
  [self lineX0: [self convertCoordX: x1]
	Y0: [self convertCoordY: y1]
	X1: [self convertCoordX: x2]
	Y1: [self convertCoordY: y2] Width: 1 Color: c]; 
  return self;
}

// places object on map
-dblPutObject: anObject atX: (double) x Y: (double) y { 
  [objects putObject: anObject atX: [self convertCoordX: x]
	   Y: [self convertCoordY: y]];
  return self;
}

// restores elevation color to a location
-dblRestoreElevationAtX: (double) x Y: (double) y {
  int x1, y1;             // integer representations of points
  x1 = [self convertCoordX: x];
  y1 = [self convertCoordY: y];
  [self drawPointX: x1 Y: y1 Color: [elevations getValueAtX: x1 Y: y1]];
  return self;
}


// conversions from map coordinates to raster coordinates
-(int) convertCoordX: (double) x {
  return ((int)((x - xsw)/metersPerSquare));
}

-(int) convertCoordY: (double) y {
  return (elevationsSizeY - (int)((y - ysw)/metersPerSquare) - 1);
}

// conversions from raster coordinates to map coordinates
-(double) convertGridX: (int) x {
  return ((double)(x * metersPerSquare)) + xsw;
}

-(double) convertGridY: (int) y {
  return ((double)((elevationsSizeY - y - 1) * metersPerSquare)) + ysw;
}

// get size of display raster
-(unsigned) getWidth {
  return width;
}

-(unsigned) getHeight {
  return height;
}

-acceptInputFor: area Called: (const char*)areaName UsingWidget: inputWidget {
  // id instructions;
  id <String> text;

  text = [String create: [self getZone]];
  [text setC: "Use the map to enter the polygons included in and"
              " excluded from "];
  [text catC: areaName];
  [text catC: "."];
  //instructions = [TkMessage createParent: inputWidget];
  [globalTkInterp eval: "
      message %s.inputInstructions -width 200 -text \"%s\"", 
	  [inputWidget getWidgetName], [text getC]];
  [globalTkInterp eval: "pack %s.inputInstructions",
		  [inputWidget getWidgetName]];
  //  [instructions setMessageText: [text getC]];
  //  [instructions packFill];
  [area setDisplayMap: self];
  [area setInputWidget: inputWidget];
  [area acceptInput];
  while ([area getDone] == 0) {
    GUI_EVENT_SYNC();
  }
  [globalTkInterp eval: "pack unpack %s.inputInstructions",
		  [inputWidget getWidgetName]];
  [text drop];
  
  return self;
}

-setButtonReleaseClient: c Message: (SEL)s {
  buttonReleaseClient = c;
  buttonReleaseSel = s;
  return self;
}

-setDoubleClickClient: c Message: (SEL)s {
  doubleClickClient = c;
  doubleClickSel = s;
  return self;
}

-handleButtonRelease: (int)n X: (int)x Y: (int)y {
  if (buttonReleaseClient && buttonReleaseSel)
    [buttonReleaseClient perform: buttonReleaseSel 
			 with: (void *)x with: (void *)y];
  return self;
}
  
-handleDoubleClick: (int)n X: (int)x Y: (int)y {
  if (doubleClickClient && doubleClickSel)
    [doubleClickClient perform: doubleClickSel 
			 with: (void *)x with: (void *)y];
  return self;
}
-displayAll {
  id r,sr;

  r = [milMap getRestricted];
  sr = [milMap getSeverelyRestricted];
  [elevDisplay display];
  if (r) {
    [r setDisplayMap: self];
    [r drawSelfOn: self Fill: Hatched];
  }
  if (sr) {
    [sr setDisplayMap: self];
    [sr drawSelfOn: self Fill: Cross];
  }
  [self drawGridLines];
  [self drawSelf];
  return self;
}

-getObjectNearX: (int) x Y: (int) y Error: (int) e {
// checks +- e units from x,y until an object is found

  id unit;
  int dist;         // distance from click to check
  id o;             // object display from the map
  int xmin, xmax;
  int ymin, ymax;
  int i,j;

  // look in vicinity of the click for a unit
  unit = [objects getObjectAtX: x Y: y];

  if (unit == nil) {
    for (dist = 1; dist <=e; dist++) {
      xmin = x - dist;
      if (xmin < 0)
	xmin = 0;
      xmax = x + dist;
      if (xmax >= [objects getSizeX])
	xmax = [objects getSizeX];
      ymin = y - dist;
      if (ymin < 0)
	ymin = 0;
      ymax = y + dist;
      if (ymax >= [objects getSizeY])
	ymax = [objects getSizeY];
      for (j = ymin; j < ymax; j++) {
	for (i = xmin; i < xmax; i++) {
	  unit = [objects getObjectAtX: i Y: j];
	  if (unit)
	    break;
	}
	if (unit)
	  break;
      }
      if (unit)
	break;
    }
  }
  return unit;
}

-drawGridLines {
  double maxSizeX;
  double maxSizeY;
  double x;
  double y;
  double linex;
  double liney;
  char buffer[256];
  double gridSize;

  gridSize = 1000;

  maxSizeX = xsw + metersPerSquare * elevationsSizeX;
  maxSizeY = ysw + metersPerSquare * elevationsSizeY;

  for (x = xsw; x < maxSizeX; x+= gridSize) {
    linex = x - fmod(x, gridSize) + gridSize;
    [self dblDrawLineFromX: linex Y: ysw ToX: linex Y: maxSizeY Color: 208];
    sprintf(buffer, "%d", (int)(fmod(linex, 100000)/gridSize));
    [self dblDrawText: buffer AtX: linex Y: ysw + 100 Color: 208];
  }
  for (y = ysw; y < maxSizeY; y+= gridSize) {
    liney = y - fmod(y, gridSize) + gridSize;
    [self dblDrawLineFromX: xsw Y: liney ToX: maxSizeX Y: liney Color: 208];
    sprintf(buffer, "%d", (int)(fmod(liney, 100000)/gridSize));
    [self dblDrawText: buffer AtX: xsw+100 Y: liney Color: 208];
  }
  return self;
}

@end




#ifndef _WIN32
typedef struct raster_private {
  GC gc;
  Tk_Window tkwin;
  X11Pixmap pm;
  X11Pixmap oldpm;
  BOOL privateColormapFlag;
} raster_private_t;

void
tkobjc_draw_text (Raster *raster,
                    int x0, int y0,
                    const char * text, 
                    Color color)
{
  raster_private_t *private = raster->private;
  PixelValue *map = ((Colormap *) raster->colormap)->map;
  Display *display = Tk_Display (private->tkwin);
  GC gc = private->gc;

  XSetForeground (display, gc, map[color]);
  XDrawString (display, private->pm, gc, x0, y0, text, strlen(text));
}

#else
typedef struct raster_private {
  GC gc;
  Tk_Window tkwin;
  dib_t *oldpm;
  dib_t *pm;
} raster_private_t;


void
tkobjc_draw_text (Raster *raster,
          int x0, int y0,
	  const char * text,
	  Color color)
{
  raster_private_t *private = raster->private;
  dib_t *dib = private->pm;
  int n;
  long hFont, hOldFont;
  HPEN oldPen, pen;
  BYTE red, green, blue;

  dib_get_color (dib, raster, (unsigned) color, &red, &green, &blue);
  pen = CreatePen (PS_SOLID, 1, RGB (red, green, blue));
  oldPen = SelectObject (dib->sourceDC, pen);

  hFont = CreateFont(0, 0, 0, 0, FW_BOLD, 0, 0, 0, SYMBOL_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Symbol");

  dib_lock (dib);
  hOldFont = SelectObject(dib->sourceDC, hFont);
  n = strlen(text);
  SetTextAlign(dib->sourceDC, TA_BASELINE);
  TextOut (dib->sourceDC, x0, y0, text, n);
  SelectObject(dib->sourceDC, hOldFont);
  DeleteObject (SelectObject (dib->sourceDC, oldPen)); 
  DeleteObject(hFont);
  dib_unlock (dib); 
}

#endif
