// TextRaster.m

#import "TextRaster.h"

#ifdef _WIN32
// Get a definition of timeval, but avoid redefinition of timezone.
#define timezone timezone__
#include <sys/time.h>
#undef timezone
// Undo X11 definition
#undef Status
#define Rectangle Win32Rectangle
#include <tkobjc/win32dib.h>
#undef BOOL
#define BOOL BOOL_
//#include "tk/tkInt.h"
#undef BOOL

#define Arguments Win32Arguments
#define Colormap X11Colormap
//#include "tk/tkWinInt.h"
#undef Colormap
#undef Arguments
#undef Rectangle
#endif



@implementation TextRaster

-drawText: (const char *) text AtX: (int) x Y:(int) y Color: (Color) c {
  tkobjc_draw_text (self,
                   x, y,
                   text, 
                   c);
  return self;
}

#ifndef _WIN32
typedef struct raster_private {
  GC gc;
  Tk_Window tkwin;
  X11Pixmap pm;
  X11Pixmap oldpm;
  BOOL privateColormapFlag;
} raster_private_t;

void
tkobjc_draw_text (Raster *raster,
                    int x0, int y0,
                    const char * text, 
                    Color color)
{
  raster_private_t *private = raster->private;
  PixelValue *map = ((Colormap *) raster->colormap)->map;
  Display *display = Tk_Display (private->tkwin);
  GC gc = private->gc;

  XSetForeground (display, gc, map[color]);
  XDrawString (display, private->pm, gc, x0, y0, text, strlen(text));
}

#else
typedef struct raster_private {
  GC gc;
  Tk_Window tkwin;
  dib_t *oldpm;
  dib_t *pm;
} raster_private_t;


void
tkobjc_draw_text (Raster *raster,
          int x0, int y0,
	  const char * text,
	  Color color)
{
  raster_private_t *private = raster->private;
  dib_t *dib = private->pm;
  int n;
  long hFont, hOldFont;
  HPEN oldPen, pen;
  BYTE red, green, blue;

  dib_get_color (dib, raster, (unsigned) color, &red, &green, &blue);
  pen = CreatePen (PS_SOLID, 1, RGB (red, green, blue));
  oldPen = SelectObject (dib->sourceDC, pen);

  hFont = CreateFont(0, 0, 0, 0, FW_BOLD, 0, 0, 0, SYMBOL_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH, "Symbol");

  dib_lock (dib);
  hOldFont = SelectObject(dib->sourceDC, hFont);
  n = strlen(text);
  SetTextAlign(dib->sourceDC, TA_BASELINE);
  TextOut (dib->sourceDC, x0, y0, text, n);
  SelectObject(dib->sourceDC, hOldFont);
  DeleteObject (SelectObject (dib->sourceDC, oldPen)); 
  DeleteObject(hFont);
  dib_unlock (dib); 
}

#endif
